#ifndef neo_ComponentID_hh_seen
#define neo_ComponentID_hh_seen

/* A file of auxiliaries that mainly distract the new reader.  */

namespace neo {
namespace cca {

/**
 * This typedef is the type everyone should use everywhere in the 
 * BuilderService and event related ports.
 * For practical purposes the memory lifecycle is as
 * follows:
 * Memory lifecycle:
 *      The framework creates items of type ComponentID_shared 
 *      and they get shoveled around to all comers by BuilderServices
 *      and related ports. 
 *      The component writer simply fetches and uses ComponentID_shared
 *      as if they are an STL type like ::std::string that is
 *      self-managing on the stack memory-wise.
 *      There is no destroyComponentID function in any interface.
 *      The user should not delete a ComponentID_shared.
 *      Where possible and sensible, ComponentID_shared items are
 *      passed by reference.
 *      Boost will take care of deleting a ComponentID on last
 *      reference going out of scope.
 *
 *      Anyone caught calling ComponentID.get() {unwrapping the shared_ptr}
 *      will be summarily shot.
 *      As ComponentID_shared is never cast to anything else, the oddity of
 *      the shared_ptr dynamic_cast does not impinge on the component writer.
 */
typedef boost::shared_ptr < ComponentID > ComponentID_shared;

/** An opaque handle for a component. 
 * Noone in component-land should ever directly mention ComponentID.
 * Use ComponentID_shared instead.
 * Memory lifecycle:
 * All ComponentID_shared are created by the framework and then shared liberally. 
 * All should be declared as local stack variables or stored in STL containers.
 * NOTE classic transition:
 * There is no toString function. Use getInstanceName() or
 * getInstanceName().c_str() instead.
 */
class ComponentID
{
public:
  
  /** required destructor. */
  virtual ~ComponentID(){}

  /** return the name. */
  virtual ::std::string getInstanceName() throw (Exception )= 0;

  /** return a framework specific serialization. */
  virtual ::std::string getSerialization() throw (Exception )= 0;

  /** ComponentID identity may not be testable
   * as a pointer comparison. All ids originate in the framework, however.
   **/
  virtual bool isSameAs(ComponentID_shared cid)= 0;

}; // end interface ComponentID

//===================================================================

}  // end namespace cca
}  // end namespace neo

#endif // neo_ComponentID_hh_seen
