#ifndef neo_ConnectionID_hh_seen
#define neo_ConnectionID_hh_seen

/* A file of auxiliaries that mainly distract the new reader.  */

namespace neo {
namespace cca {


/**
 * This typedef is the type everyone should use everywhere in the 
 * BuilderService and event related ports.
 * For practical purposes the memory lifecycle is as
 * follows:
 * Memory lifecycle:
 *      The framework creates items of type ConnectionID 
 *      and they get shoveled around to all comers by BuilderServices
 *      and related ports. 
 *      The component writer simply fetches and uses ConnectionIDs
 *      as if they are an STL type like ::std::string that is
 *      self-managing on the stack memory-wise.
 *      There is no destroyConnectionID function in any interface.
 *      Where possible and sensible, ConnectionID items are
 *      passed by reference.
 *      Boost will take care of deleting a ConnectionID on last
 *      reference going out of scope.
 *
 *      Anyone caught calling ConnectionID.get() {unwrapping the shared_ptr}
 *      will be summarily shot.
 *      As ConnectionID is never cast to anything else, the oddity of
 *      the shared_ptr dynamic_cast does not impinge on the component writer.
 */
typedef boost::shared_ptr < ConnectionID > ConnectionID_shared;

/**
 * This interface describes a CCA connection between components.
 * Component writers should never defining anything as 
 * ConnectionID *connID;
 * Rather, they should declare:
 * ConnectionID_shared connID;
 * and make calls on it as connID->getWhatever().
 *
 * A connection is made at the users direction
 * when one component provides a Port that another component
 * advertises for and uses.  The components are referred to by their
 * opaque ComponentID references and the Ports are referred to
 * by their string instance names.
 *
 * Memory lifecycle:
 * All connectionid are created by the framework and then shared
 * liberally. 
  */
class ConnectionID  
{
public:
  
  /** required destructor */
  virtual ~ConnectionID(){}

  /** Recall that ComponentID is also a shared_ptr type.
   */
  virtual ComponentID_shared getProvider() throw (Exception )= 0;

  /** Recall that ComponentID_shared is also a shared_ptr type.
     @throws a Exception if the underlying connection is no longer valid.
     */
  virtual ComponentID_shared getUser() throw (Exception )= 0; 

  virtual ::std::string getProviderPortName() throw (Exception )= 0; 

  virtual ::std::string getUserPortName() throw (Exception )= 0; 

} ; // end interface ConnectionID


}  // end namespace cca
}  // end namespace neo

#endif // neo_ConnectionID_hh_seen
