#ifndef neo_Exception_hh_seen
#define neo_Exception_hh_seen

/* A file of auxiliaries that mainly distract the new reader.  */

namespace neo {
namespace cca {

//===================================================================
/** Types of cca framework exception so far, other than typemismatch.
    Note the names here must never overlap those of Type in the same
    namespace because of the way enums work in c++.
    If we choose to change it from an enum to a java style
    all static int class, it would be a bit less safe and a bit
    less cryptic in the compiler/debugger messages.
 */
enum ExceptionType {
  Unexpected          = -1,
  Nonstandard         = 1,
  PortNotDefined      = 2,
  PortAlreadyDefined  = 3,
  PortNotConnected    = 4,
  PortNotInUse        = 5,
  UsesPortNotReleased = 6,
  BadPortName         = 7,
  BadPortType         = 8,
  BadProperties       = 9,
  BadPortInfo         = 10,
  OutOfMemory         = 11,
  NetworkError        = 12
};

//===================================================================

/** This class extends the STL std::exception to allow the thrower
 * to specify the message that the stl says we return via what().
 * By default stl says strings come out but provides no way to
 * put them into its base class exception.
 * It is isomorphic to ccafeopq::Exception, but not castable thereunto,
 * because that would make the spec framework specific. ccafeopq
 * could be changed to inherit this.
 *
 * Memory Lifecycle:
 * This exception is thrown with throw new ccafeopq::exception(...) .
 * The catcher, if any, is responsible for calling delete exceptionInstance;
 * or rethrowing if appropriate.
 */
class Exception : public ::std::exception
{
private:
	enum ExceptionType ccaType;
	::std::string  ccaMessage;

public:
	virtual ~Exception() throw (){}
	virtual const char *what() { return ccaMessage.c_str(); }

	Exception(enum ExceptionType t, const ::std::string & message) throw ()
	{ ccaType = t; ccaMessage = message; }

	Exception(enum ExceptionType t) throw ()
	{ ccaType = t; ccaMessage = ::std::string(""); }

	Exception() throw ()
	{ ccaType = Nonstandard; ccaMessage = ::std::string(""); }

	Exception(const ::std::string& message) throw ()
	{ ccaType = Nonstandard; ccaMessage = message; }

	enum ExceptionType getType() { return ccaType; }

	/** string form of std::exception::what(). */
	::std::string getMessage() {return ccaMessage; }

	void setType( enum ExceptionType t) { ccaType = t; }

	void setMessage(const ::std::string & message) { ccaMessage = message; }

};


}  // end namespace cca
}  // end namespace neo

#endif // neo_Exception_hh_seen

