#ifndef __neo_Services_hh_seen
#define __neo_Services_hh_seen

namespace neo {
namespace cca {

/** The handle through which 
* the component communicates with its containing framework.
* Memory lifecycle:
* The framework creates and destroys the Services implementation
* object outside the component's lifetime. The component may 
* store or pass its Services pointer to any other code safely
* so long as the receiving function does not store the pointer
* beyond the component's lifetime.
*/
class Services
{

public:
  
  /** Obligatory virtual destructor. */
  virtual ~Services(){}
  
  /** Fetch a port from the framework. Normally this means a uses port.
    If no uses port is connected and a provided port of the name requested
    is available, the component will be handed back that port.
    The framework will not invalidate the connection between eetPort and
    the matching releasePort.
    Returns NULL if no provided port or uses port connection matches name.
    @throw Exception if something unexpected by the framework happens,
           such as the name given being of no known port.
  */
  virtual Port * getPort(const ::std::string & name) throw(Exception )= 0;

  /** As getPort, but the framework is not permitted to block
   * execution and do possibly complex internal things to satisfy
   * the request.
  */
  virtual Port * getPortNonBlocking(const ::std::string & name) throw(Exception )= 0;

  /** Free's the connection indicated by the name 
    for modification by the component's containing framework.
    After this call any port pointer previously assigned from
    a call to getPort(name) will not be valid for use until getPort(name) 
    is called again.
    @throw Exception if something unexpected by the framework happens,
           such as the name given being of no known port.
   */
  virtual void releasePort(const ::std::string & name) throw(Exception )= 0;
  
  /** Notifies the framework that a port of specified name and type
      may be used by this component. 
    @throw Exception if something unexpected by the framework happens,
           such as the same name being registered with different types.
  */
  virtual void registerUsesPort( const ::std::string & name,  const ::std::string & type) throw(Exception )= 0;

  /** Convenience function. As registerUsesPort, 
   * but properties argument is included.
   * The properties object should be obtained from
   * Services->createTypeMap();
   * MIN_CONNECTIONS (int >= 0, default 0)
   * MAX_CONNECTIONS (int >= 1, default 1)
   * ABLE_TO_PROXY   (true,false; default false)
   */
  virtual void registerUsesPort( const ::std::string & name,  const ::std::string & type, TypeMap_shared properties) throw(Exception )= 0;


  /** Notify the framework that use of a Port, previously
      registered by this component, is no longer needed.
      @throw Exception if the port is still in use (i.e. getPort(name)
      has not had a matching releasePort(name)). 
  */
  virtual void unregisterUsesPort(const ::std::string & name) throw(Exception )= 0;

  /** Exports a Port implemented by this component to the framework.  
    This Port is now available for the framework to
    connect to other components. 
    @throw Exception if addProvidesPort fails,
      for example, because that name is already provided but with 
      a different pointer.
  */
  virtual void addProvidesPort( Port * inPort, const ::std::string & name, const ::std::string & type) throw (Exception )= 0;
  /** Convenience function. As addProvidesPort(inport,n,t),
   * but properties argument is included.
   * The properties object should be obtained from
   * Services->createTypeMap();
   * MIN_CONNECTIONS (int >= 0, default 0)
   * MAX_CONNECTIONS (int >= 1, default 1)
   * ABLE_TO_PROXY   (true,false; default false)
   */
  virtual void addProvidesPort( Port * inPort, const ::std::string & name, const ::std::string & type, TypeMap_shared properties) throw (Exception )= 0;

  /** Notifies the framework that a previously exported Port is no longer 
      available for use.
      @throw Exception if a connection to the provided port still exists
         and a getPort binding it is still in effect.
    */
  virtual void removeProvidesPort(const ::std::string & name)  throw (Exception )= 0;
  
  /** Get a reference to the component which this 
      Services object belongs.
   */
  virtual ComponentID_shared getComponentID()= 0;

  /** Construct and return a shared_ptr<> to a TypeMap. */
  virtual TypeMap_shared createTypeMap() throw (Exception )= 0;

  /** Fetch the framework's idea of the properties for a port
   * already defined in the component that this Services belongs to.
   * Modifications to the returned object will not affect the framework.
   */
  virtual TypeMap_shared getPortProperties(const ::std::string & portName)  throw (Exception )= 0;

  /** Construct a unique port name derived from the base name given.
   */
  virtual ::std::string genUniquePortName(const ::std::string & base)= 0;

  /** Obtain a callback for component destruction.
   * @param callBack an object that implements the ComponentRelease
   * interface that will be called when the component is to be destroyed.
   *
   * Register a callback to be executed when the component is going
   * to be destroyed.  During this callback, the Services object passed
   * through setServices will still be valid, but after all such
   * callbacks are made for a specific component, subsequent usage
   * of the Services object is not allowed/is undefined.
   */
  virtual void registerForRelease(ComponentRelease *callBack) throw(Exception)= 0;

};

}  // end namespace cca
}  // end namespace neo

#endif // __neo_Services_hh_seen
