#ifndef neo_TypeMap_hh_seen
#define neo_TypeMap_hh_seen

/* A file of auxiliaries that mainly distract the new reader.  */

namespace neo {
namespace cca {

/** This (TypeMap_shared) is what everyone declares in
 * component code and noone ever calls new on. 
 * TypeMap_shared is obtained from Services::createTypeMap
 * or as the return from some other function.
 */
typedef boost::shared_ptr<TypeMap> TypeMap_shared;

/** A CCA typed hash map.  Maps a string key to a particular value. Types are
    strictly enforced.  For example, values places into the map
    using putInt can be retrieved only using getInt.  Calls to
    getLong, getString, getIntArray and other get methods will
    fail (i.e. return the default value) if the value stored is of type Int.
 */
class TypeMap
{
public:
  
  /** required destructor */
  virtual ~TypeMap(){}
  
  /** Create an exact copy of this Map. */
  virtual TypeMap_shared cloneData()= 0;
  
  /** Create a new Map with no key/value associations. */
  virtual TypeMap_shared cloneEmpty()= 0;
  
  /** Get the Int value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual int32_t      getInt(const ::std::string & key,  int32_t defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the Long value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual int64_t  getLong(const ::std::string & key,  int64_t defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the Float value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual float    getFloat(const ::std::string & key,  float defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the Double value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual double   getDouble(const ::std::string & key,  double defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the Fcomplex value associated with a given key.  If the key
      was not found, return complex the value defaultValue. */
  virtual ::std::complex<float> getFcomplex(const ::std::string & key,  ::std::complex<float> defaultValue) throw (TypeMismatchException )= 0; 
  
  /** Get the Dcomplex value associated with a given key.  If the key
      was not found, return the complex value defaultValue. */
  virtual ::std::complex<double> getDcomplex(const ::std::string & key,  ::std::complex<double> defaultValue) throw (TypeMismatchException )= 0; 
  
  /** Get the string value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::string  getString(const ::std::string & key,  ::std::string defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the boolean value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual bool  getBool(const ::std::string & key,  bool defaultValue) throw (TypeMismatchException )= 0;
  
    // ======================================================
  
  /** Get the value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::vector< int32_t > getIntArray(const ::std::string & key,  ::std::vector< int32_t >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::vector< int64_t > getLongArray(const ::std::string & key,  ::std::vector< int64_t >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::vector< float >    getFloatArray(const ::std::string & key,  ::std::vector< float >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::vector< double > getDoubleArray(const ::std::string & key,  ::std::vector< double >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the complex value associated with a given key.  If the key
      was not found, return the complex value defaultValue. */
  virtual ::std::vector< ::std::complex<float> > getFcomplexArray(const ::std::string & key,  ::std::vector< ::std::complex<float> >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the complex value associated with a given key.  If the key
      was not found, return the complex value defaultValue. */
  virtual ::std::vector< ::std::complex<double> > getDcomplexArray(const ::std::string & key,  ::std::vector< ::std::complex<double> >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the value associated with a given key.  If the key
      was not found, return the value defaultValue. */
  virtual ::std::vector< ::std::string > getStringArray(const ::std::string & key,  ::std::vector< ::std::string >&  defaultValue) throw (TypeMismatchException )= 0;
  
  /** Get the value associated with a given key.  If the key
    was not found, return the value defaultValue. */
  virtual ::std::vector< bool > getBoolArray(const ::std::string & key,  ::std::vector< bool >&  defaultValue) throw (TypeMismatchException )= 0;


    // ======================================================
    
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putInt(const ::std::string & key,  int value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putLong(const ::std::string & key,  int64_t value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putFloat(const ::std::string & key,  float value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putDouble(const ::std::string & key,  double value)= 0;
      
  /** Assign a key and complex value.
      Any complex value previously assigned will be overwritten.  */
  virtual void putFcomplex(const ::std::string & key,  ::std::complex<float> value)= 0; 
      
  /** Assign a key and complex value. 
      Any complex value previously assigned will be overwritten. */
  virtual void putDcomplex(const ::std::string & key,  ::std::complex<double> value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putString(const ::std::string & key, const ::std::string & value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.  */
  virtual void putBool(const ::std::string & key,  bool value)= 0;
  
      // ======================================================
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.
      Arrays are stored by COPY, not by reference. */
  virtual void putIntArray(const ::std::string & key,  int * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putIntArray(const ::std::string & key,  ::std::vector< int >&  value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.
      Arrays are stored by COPY, not by reference. */
  virtual void putLongArray(const ::std::string & key,  int64_t * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putLongArray(const ::std::string & key,  ::std::vector< int64_t >&  value )= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.
      Arrays are stored by COPY, not by reference. */
  virtual void putFloatArray(const ::std::string & key,  float * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putFloatArray(const ::std::string & key,  ::std::vector< float >&  value )= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical.
      Arrays are stored by COPY, not by reference. */
  virtual void putDoubleArray(const ::std::string & key,  double * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putDoubleArray(const ::std::string & key,  ::std::vector< double >&  value)= 0;
      
  /** Assign a key and complex array value. Any value previously assigned
      to the same key will be overwritten if type identical.
      Complex arrays are stored by COPY.*/
  virtual void putFcomplexArray(const ::std::string & key,  ::std::complex<float> * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putFcomplexArray(const ::std::string & key,  ::std::vector< ::std::complex<float> >&  value)= 0;
      
  /** Assign a key and complex array value. Any value previously assigned
     to the same key will be overwritten if type identical.
     Complex arrays are stored by COPY. */
  virtual void putDcomplexArray(const ::std::string & key,  ::std::complex<double> * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putDcomplexArray(const ::std::string & key,  ::std::vector< ::std::complex<double> >&  value)= 0;
      
  /** Assign a key and value. Any value previously assigned
      to the same key will be overwritten if type identical..
      Arrays are stored by COPY, not by reference. */
  virtual void putStringArray(const ::std::string & key,  ::std::vector< ::std::string >&  value)= 0;
      
  /** Assign a key and value. Any value previously assigned
    to the same key will be overwritten if type identical. 
    Arrays are stored by COPY, not by reference. */
  virtual void putBoolArray(const ::std::string & key,  bool * value ,int length)= 0;
  /** c++ convenience function. */
  virtual void putBoolArray(const ::std::string & key,  ::std::vector< bool >&  value)= 0;

    // ======================================================

  /** Make the key and associated value disappear from the object. */
  virtual void remove (const ::std::string & key)= 0;

  /** Get all the names associated with a particular type
    without exposing the data implementation details.  The keys
    will be returned in an arbitrary order. If type specified is
    NoType (no specification) all keys of all types are returned.
    @see neo.cca.Type
  */
  virtual ::std::vector< ::std::string > getAllKeys( enum Type t)= 0;
  
  /** Return true if the key exists in this map */
  virtual bool hasKey(const ::std::string & key)= 0;

  /** Return the type of the value associated with this key */
  virtual enum Type typeOf(const ::std::string & key)= 0;

} ; // end interface TypeMap


}  // end namespace cca
}  // end namespace neo

#endif // neo_TypeMap_hh_seen
