#ifndef neo_ParameterPortFactory_h_seen
#define neo_ParameterPortFactory_h_seen

namespace neo {
namespace cca {
namespace ports {


/** This class mostly follows the sidl documentation
 * for the class of the same name in the gov.cca namespace.
 */
class ParameterPortFactory : public virtual neo::cca::Port
{
public:
	
	virtual ~ParameterPortFactory() {}

	/** Initialize the portData ...
	 */
	virtual void createParameterPort(neo::cca::TypeMap_shared portData, const ::std::string & portName)= 0;

	/** Define the window title for the parameter dialog.
	 **/
	virtual void setBatchTitle(neo::cca::TypeMap_shared portData, const ::std::string & title)= 0;
	
	/** Define the next tab/group title to use. All
	 * addRequest subsequent calls will add to this group.
	 * Multiple dialog tabs/groups can be defined in this way.
	 */
	virtual void setGroupName(neo::cca::TypeMap_shared portData, const std::string & newGroupName)= 0;
	virtual void setGroupNameAndTitle(neo::cca::TypeMap_shared portData, const std::string & newGroupName, const std::string & title)= 0;

	/** Define a boolean parameter and its default state.
	 */
	virtual void addRequestBoolean(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					bool deflt)= 0;

	/** Define a int parameter and its default state.
	 */
	virtual void addRequestInt(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					int deflt,
					int low,
					int high)= 0;

	/** Define a long parameter and its default state.
	 */
	virtual void addRequestLong(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					int64_t deflt,
					int64_t low,
					int64_t high)= 0;

	/** Define a float parameter and its default state.
	 * The configured value is always available by
	 * portData->getFloat(name, ...) and it will be
	 * in the range [low, high].
	 */
	virtual void addRequestFloat(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					float deflt,
					float low,
					float high)= 0;

	/** Define a double parameter and its default state.
	 */
	virtual void addRequestDouble(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					double deflt,
					double low,
					double high)= 0;

	/** Define a string parameter and its default state.
	 */
	virtual void addRequestString(neo::cca::TypeMap_shared portData,
					const std::string & name, 
					const std::string & help,
					const std::string & prompt,
					const std::string & deflt)= 0;
	/** define a new choice for a string parameter. */
	virtual void addRequestStringChoice(neo::cca::TypeMap_shared portData,
			const std::string & key, const std::string &choice)= 0;

	/** define more choices for a string parameter. */
	virtual void addStringChoices(neo::cca::TypeMap_shared portData,
		                        const std::string & key,
					std::vector< std::string > choices)= 0;

	/** define more choices for a int32 parameter. */
	virtual void addIntChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< int32_t > choices)= 0;

	/** define more choices for a int32 parameter, raw.
	 * nChoices is number of int elements in choices, not memory size. */
	virtual void addIntChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		int32_t choices[], size_t nChoices)= 0;
	
	/** define more choices for a int64 parameter. */
	virtual void addLongChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< int64_t > choices)= 0;

	/** define more choices for a int64 parameter, raw. 
	 * nChoices is number of long elements in choices, not memory size. */
	virtual void addLongChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		int64_t choices[], size_t nChoices)= 0;

	/** define more choices for a parameter. */
	virtual void addFloatChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< float > choices)= 0;

	/** define more choices for a parameter, raw. 
	 * nChoices is number of long elements in choices, not memory size. */
	virtual void addFloatChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		float choices[], size_t nChoices)= 0;

	/** define more choices for a parameter. */
	virtual void addDoubleChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< double > choices)= 0;

	/** define more choices for a parameter, raw. 
	 * nChoices is number of long elements in choices, not memory size. */
	virtual void addDoubleChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		double choices[], size_t nChoices)= 0;

	/** define more choices for a parameter. */
	virtual void addFcomplexChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< std::complex< float > > choices)= 0;

	/** define more choices for a parameter, raw. 
	 * nChoices is number of long elements in choices, not memory size. */
	virtual void addFcomplexChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::complex< float > choices[], size_t nChoices)= 0;

	/** define more choices for a parameter. */
	virtual void addDcomplexChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::vector< std::complex< double > > choices)= 0;

	/** define more choices for a parameter, raw. 
	 * nChoices is number of long elements in choices, not memory size. */
	virtual void addDcomplexChoices(neo::cca::TypeMap_shared portData,
		const std::string & key,
		std::complex< double > choices[], size_t nChoices)= 0;


	 /** Clear all previously added requests, titles, groups. After
	  *  this call, it is as if the ParameterPort has
	  *  been created but never configured. The values of
	  *  previously defined parameters will, nonethesless,
	  *  remain in the typemap. (They may have been in the
	  *  map before it was used as a parameter port, after all.)
	  *  Typically, this is used only by someone implementing
	  *  the updateParameterPort function from
	   *  class ParameterGetListener.
	  */
	virtual void clearRequests(neo::cca::TypeMap_shared portData)= 0;

	/** Register listener (the component) that wishes to have
	 * a chance to change the contents of its ParameterPort
	 * just before the parameters typemap is used to
	 * render the parameter dialog.
	 * @param powner a pointer to the listener that will be
	 * forgotten when it is no longer needed. 
	 * @param portData the component's private parameter data map.
	 */
	virtual void registerUpdater(neo::cca::TypeMap_shared portData, 
				ParameterGetListener *powner)= 0;

	/** Register listener (the component) if it wishes to be
	 * informed when an parameter is set.
	 * Listeners are called after values are set.
	 * @param powner a pointer to the listener that will be
	 * forgotten when it is no longer needed.
	 * @param portData the component's private parameter data map.
	 */
	virtual void registerUpdatedListener(neo::cca::TypeMap_shared portData,
				ParameterSetListener *powner)= 0;

	/** Signal that the ParameterPort is fully defined and should
	 * now pop out on the component. The Services passed here
	 * must be the component's own Services handle.
	 */
	virtual void addParameterPort(neo::cca::TypeMap_shared portData, neo::cca::Services * svc)= 0;

	/** Cause a previously defined parameter port to go away. */
	virtual void removeParameterPort(neo::cca::TypeMap_shared portData, neo::cca::Services * svc)= 0;


}; // end virtual interface ParameterPortFactory


}  // end namespace ports
}  // end namespace cca
}  // end namespace neo
#endif // neo_ParameterPortFactory_h_seen
